import java.io.*;
import java.security.*;

public class SignIt {
  static private byte [] loadByteData (String filename) {
    FileInputStream fis = null;
    try {
      fis = new FileInputStream (filename);
      BufferedInputStream bis = new BufferedInputStream (fis);
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      int ch;
      while ((ch = bis.read()) != -1) {
        baos.write (ch);
      }
      return baos.toByteArray();
    } catch (IOException e) {
      if (fis != null) {
        try {
          fis.close();
        } catch (IOException ee) {
          // ignore
        }
      }
      return null;
    }
  }
  static private String hexDigit(byte x) {
    StringBuffer sb = new StringBuffer();
    char c;
    // First nibble
    c = (char) ((x >> 4) & 0xf);
    if (c > 9) {
      c = (char) ((c - 10) + 'a');
    } else {
      c = (char) (c + '0');
    }
    sb.append (c);
    // Second nibble
    c = (char) (x & 0xf);
    if (c > 9) {
      c = (char)((c - 10) + 'a');
    } else {
      c = (char)(c + '0');
    }
    sb.append (c);
    return sb.toString();
  }
  static private byte[] computeDigest (Signature sig, PrivateKey priv, byte[] content) {
    try {
      if (content != null) {
        sig.initSign(priv);
        sig.update (content);
        byte digest[] = sig.sign();
        return digest;
      } else {
        return null;
      }
    } catch (InvalidKeyException e) {
        return null;
    } catch (SignatureException e) {
        return null;
    }
  }
  static public boolean verifyDigest (Signature sig, PublicKey pub, byte[] content, byte[] digest) {
    try {
      if (content != null) {
        sig.initVerify(pub);
        sig.update (content);
        return sig.verify(digest);
      } else {
        return false;
      }
    } catch (InvalidKeyException e) {
        return false;
    } catch (SignatureException e) {
        return false;
    }
  }
  static private void printUsage () {
      System.err.println ("java SignIt filename(s)");
  }
  public static void main (String args[]) {
    if (args.length < 1) {
      printUsage();
      System.exit (-1);
    }
    KeyPairGenerator kpg = null;
    try {
      kpg = KeyPairGenerator.getInstance("DSA");
    } catch (NoSuchAlgorithmException e) {
      System.err.println ("Invalid generator algorithm");
      printUsage();
      System.exit (-2);
    }
    kpg.initialize(1024);
    KeyPair kp = kpg.generateKeyPair();
    Signature sig = null;
    try {
      sig = Signature.getInstance("DSA");
    } catch (NoSuchAlgorithmException e) {
      System.err.println ("Invalid signature algorithm");
      printUsage();
      System.exit (-3);
    }
    PrivateKey priv = kp.getPrivate();
    PublicKey pub = kp.getPublic();
    int argsLength = args.length;
    for (int i=0;i<argsLength;i++) {
      byte content[] = loadByteData (args[i]);
      byte signed[] = computeDigest (sig, priv, content);
      boolean verifies = verifyDigest (sig, pub, content, signed);
      System.out.println ("filename: " + args[i]);
      StringBuffer hexString = new StringBuffer();
      int signedLength = signed.length;
      for (int j=0;j<signedLength;j++) {
        hexString.append (hexDigit(signed[j]));
        hexString.append (" ");
      }
      System.out.println (hexString.toString());
      System.out.println(verifies);
    }
  }
}
