import java.io.*;
import java.security.*;

public class SignIt2V {
  static private String hexDigit(byte x) {
    StringBuffer sb = new StringBuffer();
    char c;
    // First nibble
    c = (char) ((x >> 4) & 0xf);
    if (c > 9) {
      c = (char) ((c - 10) + 'a');
    } else {
      c = (char) (c + '0');
    }
    sb.append (c);
    // Second nibble
    c = (char) (x & 0xf);
    if (c > 9) {
      c = (char)((c - 10) + 'a');
    } else {
      c = (char)(c + '0');
    }
    sb.append (c);
    return sb.toString();
  }
  static public boolean verifyDigest (Signature sig, PublicKey pub, byte[] content, byte[] digest) {
    try {
      if (content != null) {
        sig.initVerify(pub);
        sig.update (content);
        return sig.verify(digest);
      } else {
        return false;
      }
    } catch (InvalidKeyException e) {
        return false;
    } catch (SignatureException e) {
        return false;
    }
  }
  static private void printUsage () {
      System.err.println ("java SignIt2V");
  }
  public static void main (String args[]) throws IOException, ClassNotFoundException {
    Signature sig = null;
    try {
      sig = Signature.getInstance("DSA");
    } catch (NoSuchAlgorithmException e) {
      System.err.println ("Invalid signature algorithm");
      printUsage();
      System.exit (-3);
    }
    FileInputStream fis = new FileInputStream ("msg.dat");
    ObjectInputStream ois = new ObjectInputStream (fis);
    byte content[] = (byte[])ois.readObject ();  // message
    byte signed[]  = (byte[])ois.readObject ();  // digest
    PublicKey pub  = (PublicKey)ois.readObject ();  // public key
    boolean verifies = verifyDigest (sig, pub, content, signed);
    StringBuffer hexString = new StringBuffer();
    int signedLength = signed.length;
    for (int i=0;i<signedLength;i++) {
      hexString.append (hexDigit(signed[i]));
      hexString.append (" ");
    }
    System.out.println (hexString.toString());
    System.out.println(verifies);
  }
}
