package hire;

import java.util.*;
import java.io.*;
import java.beans.*;

public class Hire implements Serializable {

  private float salary = 0;
  private int department = Dept.UNASSIGNED;
  private Vector hireListeners = new Vector();
  private PropertyChangeSupport changes =
    new PropertyChangeSupport(this);

  public Hire() {
  }

  public synchronized void
    addHireListener(HireListener l) {
      hireListeners.addElement(l);
  }

  public synchronized void
    removeHireListener(HireListener l) {
      hireListeners.removeElement(l);
  }

  protected void notifyHired() {
    Vector l;
    HireEvent h = new HireEvent(this);
    synchronized(this) {
      l = (Vector)hireListeners.clone();
    }
    for (int i = 0; i < l.size(); i++) {
      HireListener hl = (HireListener)l.elementAt(i);
      hl.hired(h);
    }
  }

  public void addPropertyChangeListener(
                      PropertyChangeListener p) {
    changes.addPropertyChangeListener(p);
  }

  public void removePropertyChangeListener(
                      PropertyChangeListener p) {
    changes.removePropertyChangeListener(p);
  }

  public void setSalary(float salary) {
    float old = this.salary;
    this.salary = salary; 
    changes.firePropertyChange(
      "salary", new Float(old), 
       new Float(salary));
//    System.out.println("Salary is: " + salary);
  }

  public float getSalary() {
    return salary;
  }

  public void setDepartment(int dept) {
    if (dept < 0 || dept > Dept.UNASSIGNED)
      return;
    int old = department;
    department = dept;
    changes.firePropertyChange(
      "department", new Integer(old), 
       new Integer(department));
//    System.out.println("Department is: " + department);
  }

  public int getDepartment() {
    return department;
  }
}