/**** BirthDate.java ****/

import java.io.*;
import java.util.*;
import java.text.*;
import javax.servlet.*;
import javax.servlet.http.*;

/*
Ex. http://localhost:8080/servlet/
      BirthDate?month=october&year=1997&day=9
*/

public class BirthDate extends HttpServlet {
  private static final String title =
    "Birth Date Information";

  public String getServletInfo() {
    return "BirthDate Servlet";
  }

  public void doGet(HttpServletRequest req,
      HttpServletResponse res)
      throws ServletException, IOException {
    res.setContentType("text/html");
    ServletOutputStream os = res.getOutputStream();
    String html = makeHeader(title);
    html += makeTable(
      req.getParameterValues("year")[0],
      req.getParameterValues("month")[0],
      req.getParameterValues("day")[0]);
    html += makeFooter();
    os.write(html.getBytes());
    os.flush();
  }

  private String makeHeader(String title) {
    return
      "<html>" +
      "<title>" + title + "</title>" +
      "<body>";
  }

  private String makeFooter() {
    return "</body>" + "</html>";
  }

  private String makeTable(String year, String month,
      String day) {
    String html = "<table border=3 cellpadding=3>";
    boolean validDate = !(getYear(year) == -1 ||
      getMonth(month) == -1 || getDay(day) == -1);
    if (!validDate) {
      html += "<tr><td><b>Invalid Date!</b></td></tr>";
      html += "</table>";
      return html;
    }
    GregorianCalendar calendar =
      new GregorianCalendar(
        getYear(year), getMonth(month), getDay(day));
    int dayOfWeek =
      calendar.get(GregorianCalendar.DAY_OF_WEEK);
    String dayOfWeekAsString =
      new DateFormatSymbols().getWeekdays()[dayOfWeek];
    String leapYear =
      calendar.isLeapYear(getYear(year)) ? "Yes" : "No";
    html += "<tr>" +
      "<td>Birth Day of Week:</td>" +
      "<td>" + dayOfWeekAsString + "</td>" +
      "</tr>" +
      "<tr>" +
      "<td>Born in a Leap Year:</td>" +
      "<td>" + leapYear + "</td>" +
      "</tr>" +
      "</table>";
    return html;
  }

  private int getYear(String year) {
    if (year == null)
      return -1;
    try {
      return Integer.parseInt(year);
    }
    catch (Exception e) {
      return -1;
    }
  }

  private int getMonth(String month) {
    if (month == null || month.length() == 0)
      return -1;
    String[] months =
      new DateFormatSymbols().getMonths();
    for (int i = 0; i < months.length; i++)
      if (month.equalsIgnoreCase(months[i]))
        return i;
    return -1;
  }

  private int getDay(String day) {
    if (day == null)
      return -1;
      int d;
    try {
      d = Integer.parseInt(day);
    }
    catch (Exception e) {
      return -1;
    }
    return (d < 1) ? -1 : d;
  }
}
